@@ -153,9 +153,6 @@ def _recreate_raw_data(f, name, versions_to_delete, tmp=False):
 
     chunks_to_keep = sorted(chunks_to_keep, key=lambda i: i.args[0].args[0])
 
-    if not chunks_to_keep:
-        return {}
-
     raw_data = f['_version_data'][name]['raw_data']
     chunks = ChunkSize(raw_data.chunks)
     new_shape = (len(chunks_to_keep)*chunks[0], *chunks[1:])
@@ -319,10 +316,17 @@ def delete_versions(f, versions_to_delete):
 
     versions = version_data['versions']
 
+    if '__first_version__' in versions_to_delete:
+        raise ValueError("Cannot delete first version")
+
     for version in versions_to_delete:
         if version not in versions:
             raise ValueError(f"Version {version!r} does not exist")
 
+    current_version = versions.attrs['current_version']
+    while current_version in versions_to_delete:
+        current_version = versions[current_version].attrs['prev_version']
+
     versions_to_keep = set(versions) - set(versions_to_delete)
 
     def delete_dataset(name):
@@ -332,7 +336,15 @@ def delete_versions(f, versions_to_delete):
         # Recreate the raw data.
         raw_data_chunks_map = _recreate_raw_data(f, name, versions_to_delete)
 
-        if not raw_data_chunks_map:
+        delete_dataset = True
+        for version_name in all_versions(f):
+            if version_name in versions_to_delete:
+                continue
+            if name in f['_version_data/versions'][version_name]:
+                delete_dataset = False
+                break
+
+        if delete_dataset:
             del version_data[name]
             return
 
@@ -361,6 +373,8 @@ def delete_versions(f, versions_to_delete):
     for version in versions_to_delete:
         del versions[version]
 
+    versions.attrs['current_version'] = current_version
+
 # Backwards compatibility
 delete_version = delete_versions
 
